@@ -95,6 +95,7 @@ var targets = map[string]target{
 			{src: "etc/linux-systemd/system/syncthing@.service", dst: "deb/lib/systemd/system/syncthing@.service", perm: 0644},
 			{src: "etc/linux-systemd/system/syncthing-resume.service", dst: "deb/lib/systemd/system/syncthing-resume.service", perm: 0644},
 			{src: "etc/linux-systemd/user/syncthing.service", dst: "deb/usr/lib/systemd/user/syncthing.service", perm: 0644},
+			{src: "etc/firewall-ufw/syncthing", dst: "deb/etc/ufw/applications.d/syncthing", perm: 0644},
 		},
 	},
 	"stdiscosrv": {
@@ -525,6 +526,8 @@ func buildDeb(target target) {
 }
 
 func buildSnap(target target) {
+	os.RemoveAll("snap")
+
 	tmpl, err := template.ParseFiles("snapcraft.yaml.template")
 	if err != nil {
 		log.Fatal(err)
@@ -534,7 +537,27 @@ func buildSnap(target target) {
 	if err != nil {
 		log.Fatal(err)
 	}
-	err = tmpl.Execute(f, map[string]string{"Version": version})
+
+	snaparch := goarch
+	if snaparch == "armhf" {
+		goarch = "arm"
+	}
+	snapver := version
+	if strings.HasPrefix(snapver, "v") {
+		snapver = snapver[1:]
+	}
+	snapgrade := "devel"
+	if matched, _ := regexp.MatchString(`^\d+\.\d+\.\d+$`, snapver); matched {
+		snapgrade = "stable"
+	}
+	err = tmpl.Execute(f, map[string]string{
+		"Version":      snapver,
+		"Architecture": snaparch,
+		"Grade":        snapgrade,
+	})
+	if err != nil {
+		log.Fatal(err)
+	}
 	runPrint("snapcraft", "clean")
 	build(target, []string{"noupgrade"})
 	runPrint("snapcraft")
